<?php
/**
 *	NP_vernetzungsfunktion Plugin fr Nucleus CMS
 *	von Bjrn Urban (http://dejure.org)
 *
 *  Description:
 *  Diese Erweiterung verlinkt automatisch zitierte Gesetze und Rechtsprechung (Aktenzeichen und Fundstellen) mit den Inhalten von dejure.org.
 * 
 *	History:
 *	1.1 - Erste Version
 *	1.2 - Zweite Version (Korrektur beim automatischen Loeschen von Cache-Eintraegen)
 */

// Fuer aeltere Nucleus-Versionen (<=2.0)
if (!function_exists('sql_table')) {
	function sql_table($name) {
		return 'nucleus_' . $name;
	}
}
define('TABELLENNAME', sql_table('plug_vernetzung_cache'));

class NP_vernetzungsfunktion extends NucleusPlugin {

	function getName() { return 'dejure.org Vernetzungsfunktion'; }
	function getAuthor()  { return 'Bjoern Urban'; }
	function getURL()  { return 'http://dejure.org/'; }
	function getVersion() { return '1.2'; }
	function getDescription() {
		return 'Die Vernetzungsfunktion verlinkt in Online-Texten automatisch Gesetzes- und Rechtsprechungszitate mit den entsprechenden Rechtsquellen.';
	}

	function getEventList()
	{
		return array('PreItem', 'PreComment', 'PreBlogContent');
	}

	function install() {
		$result = sql_query(sprintf("SELECT name, value FROM %s", sql_table('config')));
		while ($row = mysql_fetch_object($result)) {
			if ($row->name == 'SiteName') {
				$anbieterkennung = $row->value;
			}
			if ($row->name == 'AdminEmail') {
				$kontaktemail = $row->value;
			}
		}
        mysql_free_result($result);
		$this->createOption('anbieterkennung','Name Ihrer Website','text',$anbieterkennung);
		$this->createOption('kontaktemail','Kontakt-Adresse','text',$kontaktemail);
		$this->createOption('vernetzen_items','Beitraege vernetzen?','yesno','yes');
		$this->createOption('vernetzen_comments','Kommentare der Nutzer vernetzen?','yesno','yes');
		$this->createOption('target','Ziel der Links (z. B. _blank fuer ein neues Fenster)','text','');
		$this->createOption('class','CSS-Klasse zum Formatieren der Links','text','');
		$this->createOption('linkstil','Nur die Nummern der Vorschriften verlinken (ja), oder moeglichst viel davon (nein)','yesno', 'no');
		$this->createOption('cache_leeren','Cache leeren, damit Aenderungen der Einstellungen auch fuer alte Beitraege wirksam werden','yesno', 'yes');
        sql_query(sprintf("CREATE TABLE IF NOT EXISTS %s (
		  schluessel varchar(32),
		  text longtext NOT NULL,
		  gespeichert datetime NOT NULL default '0000-00-00 00:00:00',
		  UNIQUE KEY ID (schluessel)
		) TYPE=MYISAM", TABELLENNAME));
	}

	function uninstall()
	{
        sql_query(sprintf("DROP TABLE IF EXISTS  %s", TABELLENNAME));
	}

	function supportsFeature($what)
	{
		switch ($what)
		{
			case 'SqlTablePrefix':
				return 1;
			default:
				return 0;
		}
	}

	function event_PreItem($_data) {
		if ($this->getOption('vernetzen_items') == 'yes') {
			$_data['item']->body = $this->djo_vernetzen($_data['item']->body);
			$_data['item']->more = $this->djo_vernetzen($_data['item']->more);
		}
	}

	function event_PreComment($_data) {
		if ($this->getOption('vernetzen_comments') == 'yes') {
			$_data['comment']['body'] = $this->djo_vernetzen($_data['comment']['body']);
		}
	}

	function event_PreBlogContent($_data) {
		global $djo_vernetzung_in_cache_schreiben;
		$this->djo_zwischenspeicherung();
	}

	function djo_vernetzen($ausgabetext) {
		global $djoAktenzeichenIgnorieren;
		global $CONF, $blog;
		if (!preg_match("/|&sect;|&#167;|Art\.|[\/\.] ?[0-9][0-9](?![0-9\/])| [0-9][0-9]?[\/\.] ?[0-9][0-9](?![0-9\.])| [A-Z]+ [0-9][0-9]\. ?[0-9]|[0-9][0-9],/", $ausgabetext) || preg_match("/<!-- *ohnedjo *-->/", $ausgabetext)) {
			return $ausgabetext;
		}
		//$params['vernetzungscache'] = $this->getOption('vernetzungscache'); # Relativer oder absoluter Name eines (les- und beschreibbaren! - Attribute: etwa 777) Verzeichnisses, mit abschlieendem "/" (bei leerer Variablen luft die Funktion ohne Cache)
		$params['anbieterkennung'] = $this->getOption('anbieterkennung'); # Hier eine feste Bezeichnung fr Ihren Dienst zur Kommunikation mit dejure.org festlegen (Buchstaben und Zahlen erlaubt, mindestens 5 Stellen)
		$params['kontaktemail'] = $this->getOption('kontaktemail'); # Wird im Rahmen der Serverkommunikation jeweils an dejure.org bertragen und dient im brigen ausschlielich der Kommunikation bei technischen Problemen.
		$params['target'] = $this->getOption('target'); # Zum Beispiel fr ffnen in neuem Fenster: "_blank"
		$params['class'] = $this->getOption('class');
		$params['linkstil'] = 'weit';
		if ($this->getOption('linkstil') == 'yes') {
			$params['linkstil'] = 'schmal';
		}
		$params['cache_leeren'] = $this->getOption('cache_leeren');
		if ($params['cache_leeren'] == 'yes') {
			$this->setOption('cache_leeren', 'no');
			$this->djo_cache_leeren();
		}
		$rueckgabe = $this->vernetzen_ueber_cache($ausgabetext);
		if (empty($rueckgabe)) {
			$rueckgabe = $this->vernetzen_ueber_dejure_org($ausgabetext, array("Anbieterkennung" => $params['anbieterkennung']."__".$params['kontaktemail'], "target" => $params['target'], "class" => $params['class'], "version" => 'nucleus-'.$this->getVersion(), 'format' => $params['linkstil'], "AktenzeichenIgnorieren" => $djoAktenzeichenIgnorieren));
			$rueckgabe = $this->integritaetskontrolle_und_cache($ausgabetext, $rueckgabe);
		}
		return $rueckgabe;
	}

	function vernetzen_ueber_cache($ausgangstext) {
		$schluessel = strlen($ausgangstext) . '-' . substr(md5($ausgangstext), 0, -1-strlen(strlen($ausgangstext)));;
		$sql = sprintf("SELECT text FROM %s WHERE schluessel = '%s'", TABELLENNAME, mysql_escape_string($schluessel));
		$rueckgabe = sql_query($sql);
		while ($row = mysql_fetch_object($rueckgabe)) {
			$temp = $row->text;
		}
        mysql_free_result($rueckgabe);
		return $temp;
	}

	function djo_zwischenspeicherung() {
		global $djo_vernetzung_in_cache_schreiben, $djo_db_abgefragt;
		if (!is_array($djo_vernetzung_in_cache_schreiben) || count($djo_vernetzung_in_cache_schreiben) == 0) {
			return;
		}
		if (is_array($djo_vernetzung_in_cache_schreiben)) {
			foreach ($djo_vernetzung_in_cache_schreiben as $vernetzung) {
				$schluessel = strlen($vernetzung[0]) . '-' . substr(md5($vernetzung[0]), 0, -1-strlen(strlen($vernetzung[0])));
				if ($vernetzung[0] == $vernetzung[1]) {
					$text = "<!-- idem -->";
				}
				else {
					$text = $vernetzung[1];
				}
				sql_query(sprintf("REPLACE INTO %s (schluessel, text, gespeichert) VALUES ('%s', '%s', NOW());", TABELLENNAME, mysql_escape_string($schluessel), mysql_escape_string($text)));
			}
		}
		$this->djo_cache_verfall();
	}

	function djo_cache_leeren() {
        sql_query("TRUNCATE TABLE ".TABELLENNAME.";");
	}

	function djo_cache_verfall() {
		if (date("G") < 6) { // Zwischen 0 und 6 Uhr ein mal alle Dateien im Cacheverzeichnis lschen, die lter als 4 Tage sind. Dies lst regelmig eine aktualisierende Neuvernetzung aus
			sql_query("DELETE FROM ".TABELLENNAME." WHERE TO_DAYS(NOW()) - TO_DAYS(gespeichert) > 4)");
		}
	}

	function integritaetskontrolle_und_cache($ausgangstext, $neuertext) {
		global $djo_vernetzung_in_cache_schreiben, $djo_db_abgefragt;
		if (preg_replace("/<a href=\"http:\/\/dejure.org\/[^>]*>([^<]*)<\/a>/i", "\\1", $ausgangstext) == preg_replace("/<a href=\"http:\/\/dejure.org\/[^>]*>([^<]*)<\/a>/i", "\\1", $neuertext)) {
			$djo_vernetzung_in_cache_schreiben[] = array($ausgangstext, $neuertext, $djo_vernetzung_erfolgt); // Vorbereitet: Serververbindungsproblem in die DB schreiben
			return $neuertext;
		}
		else {
			return $ausgangstext;
		}
	}

	function vernetzen_ueber_dejure_org($ausgangstext, $parameter = array()) {
		// Mgliche Parameter: Anbieterkennung / Dokumentkennung / target / class / AktenzeichenIgnorieren / zeitlimit_in_sekunden
		global $djo_vernetzung_erfolgt;
		$djo_vernetzung_erfolgt = 0;
		$uebergabe = "Originaltext=".urlencode($ausgangstext);
		foreach ($parameter as $option => $wert) {
			if ($option == "zeitlimit_in_sekunden") {
				$zeitlimit_in_sekunden = $wert;
			}
			else {
				$uebergabe .= "&".urlencode($option)."=".urlencode($wert);
			}
		}
		if (!isset($zeitlimit_in_sekunden)) {
			$zeitlimit_in_sekunden = 2;
		}
		$header = "POST http://rechtsnetz.dejure.org/dienste/vernetzung/vernetzen HTTP/1.0\r\n";
		$header .= "Content-type: application/x-www-form-urlencoded\r\n";
		$header .= "Content-length: " . strlen($uebergabe) . "\r\n\r\n";

		$fp = @fsockopen("rechtsnetz.dejure.org", 80, $errno, $errstr, 12);

		if (!$fp) { // Verbindung gescheitert
			return $ausgangstext;
		}
		else {
			socket_set_timeout($fp, $zeitlimit_in_sekunden, 0); // Socket nach $zeitlimit_in_sekunden Sekunden auf jeden Fall wieder frei geben
			socket_set_blocking($fp, true);
			fputs($fp, $header.$uebergabe);
			$timeOutSock = false;
			$eofSock = false;
			$rueckgabe="";
			while (!$eofSock && !$timeOutSock) {
				$rueckgabe.= fgets($fp, 1024); //
				$stSock = socket_get_status($fp);
				$eofSock = $stSock["eof"];
				$timeOutSock = $stSock["timed_out"];
			}
			fclose($fp);
			if (!preg_match("/^(.*?)\r?\n\r?\n\r?\n?(.*)/s",$rueckgabe, $rueckgabeARR)) {
				return $ausgangstext; // Zeitberschreitung oder Verbindungsproblem
			}
			else if (strpos($rueckgabeARR[1],"200 OK")===false) {
				return $ausgangstext; // sonstiges Serverproblem
			}
			else {
				$rueckgabe = $rueckgabeARR[2];
				if (strlen($rueckgabe) < strlen($ausgangstext)) {
					return $ausgangstext;
				}
				$djo_vernetzung_erfolgt = 1;
				return $rueckgabe;
			}
		}
	}
}
?>
